@@ -21,6 +21,16 @@
 
 #include <string>
 #include <vector>
+
+/* following is required for OS X */
+
+#ifdef nil
+#undef nil
+#endif
+
+/* end OS X */
+
+#include <sigc++/slot.h>
 #include <cairomm/enums.h>
 #include <cairomm/exception.h>
 #include <cairomm/fontoptions.h>
@@ -59,6 +69,34 @@ namespace Cairo
 class Surface
 {
 public:
+  /** For example:
+   * <code>
+   * ErrorStatus my_write_func(unsigned char* data, unsigned int length);
+   * </code>
+   *
+   * This is the type of function which is called when a backend needs to write
+   * data to an output stream. It is passed the data to write and the length of
+   * the data in bytes. The write function should return CAIRO_STATUS_SUCCESS if
+   * all the data was successfully written, CAIRO_STATUS_WRITE_ERROR otherwise.
+   *
+   * @param data the buffer containing the data to write
+   * @param length the amount of data to write
+   * @return the status code of the write operation
+   */
+  typedef sigc::slot<ErrorStatus, const unsigned char* /*data*/, unsigned int /*length*/> SlotWriteFunc;
+  /**
+   * This is the type of function which is called when a backend needs to read
+   * data from an input stream. It is passed the buffer to read the data into
+   * and the length of the data in bytes. The read function should return
+   * CAIRO_STATUS_SUCCESS if all the data was successfully read,
+   * CAIRO_STATUS_READ_ERROR otherwise.
+   *
+   * @param data the buffer into which to read the data
+   * @param length the amount of data to read
+   * @return the status code of the read operation
+   */
+  typedef sigc::slot<ErrorStatus, unsigned char* /*data*/, unsigned int /*length*/> SlotReadFunc;
+
   /** Create a C++ wrapper for the C instance. This C++ instance should then be
    * given to a RefPtr.
    *
@@ -149,8 +187,57 @@ public:
    */
   void set_fallback_resolution(double x_pixels_per_inch, double y_pixels_per_inch);
 
+  /** This function returns the previous fallback resolution set by
+   * set_fallback_resolution(), or default fallback resolution if never set.
+   *
+   * @param x_pixels_per_inch horizontal pixels per inch
+   * @param y_pixels_per_inch vertical pixels per inch
+   *
+   * @since 1.8
+   */
+  void get_fallback_resolution(double& x_pixels_per_inch, double& y_pixels_per_inch) const;
+
   SurfaceType get_type() const;
 
+  /**This function returns the content type of surface which indicates whether
+   * the surface contains color and/or alpha information.
+   *
+   * @since 1.8
+   */
+  Content get_content() const;
+
+  /**
+   * Emits the current page for backends that support multiple pages,
+   * but doesn't clear it, so that the contents of the current page will
+   * be retained for the next page.  Use show_page() if you want to get an empty
+   * page after the emission.
+   *
+   * @since 1.6
+   */
+  void copy_page();
+
+  /**
+   * Emits and clears the current page for backends that support multiple pages.
+   * Use copy_page() if you don't want to clear the page.
+   *
+   * @since 1.6
+   */
+  void show_page();
+
+  /** Returns whether the surface supports sophisticated
+   * Context::show_text_glyphs() operations.  That is, whether it actually uses
+   * the provided text and cluster data to a Context::show_text_glyphs() call.
+   *
+   * Note: Even if this function returns %FALSE, a Context::show_text_glyphs()
+   * operation targeted at this surface will still succeed.  It just will act
+   * like a Context::show_glyphs() operation.  Users can use this function to
+   * avoid computing UTF-8 text and cluster mapping if the target surface does
+   * not use it.
+   *
+   * @since 1.8
+   **/
+  bool has_show_text_glyphs() const;
+
 #ifdef CAIRO_HAS_PNG_FUNCTIONS
 
   /** Writes the contents of surface to a new file filename as a PNG image.
@@ -169,9 +256,13 @@ public:
    *
    * @param write_func  The function to be called when the backend needs to
    * write data to an output stream
-   * @param closure	closure data for the write function
+   *
+   * @since 1.8
    */
-  void write_to_png(cairo_write_func_t write_func, void *closure); //TODO: Use a sigc::slot?
+  void write_to_png_stream(const SlotWriteFunc& write_func);
+
+  /** @deprecated Use write_to_png_stream instead */
+  void write_to_png(cairo_write_func_t write_func, void *closure);
 
 #endif // CAIRO_HAS_PNG_FUNCTIONS
 
@@ -211,8 +302,15 @@ protected:
   /** The underlying C cairo surface type that is wrapped by this Surface
    */
   cobject* m_cobject;
+
+private:
+  Surface(const Surface&);
+  Surface& operator=(const Surface&);
 };
 
+/** @example image-surface.cc
+ * An example of using Cairo::ImageSurface class to render to PNG
+ */
 
 /** Image surfaces provide the ability to render to memory buffers either
  * allocated by cairo or by the calling code. The supported image formats are
@@ -277,17 +375,35 @@ public:
    */
   int get_stride() const;
 
-
-  /** Creates an image surface of the specified format and dimensions. The
-   * initial contents of the surface is undefined; you must explicitely clear
-   * the buffer, using, for example, Cairo::Context::rectangle() and
-   * Cairo::Context::fill() if you want it cleared.
+  /**
+   * This function provides a stride value that will respect all
+   * alignment requirements of the accelerated image-rendering code
+   * within cairo. Typical usage will be of the form:
    *
-   * Use this function to create the surface if you don't need access to the
-   * internal data and want cairo to manage it for you.  Since you don't have
-   * access to the internal data, the resulting surface can only be saved to a
-   * PNG image file (if cairo has been compiled with PNG support) or as a
-   * source surface (see Cairo::SurfacePattern).
+   * @code
+   * int stride;
+   * unsigned char *data;
+   * Cairo::RefPtr<Cairo::ImageSurface> surface;
+   *
+   * stride = Cairo::ImageSurface::format_stride_for_width (format, width);
+   * data = malloc (stride * height);
+   * surface = Cairo::ImageSurface::create (data, format, width, height);
+   * @endcode
+   *
+   * @param format A Cairo::Format value
+   * @param width The desired width of an image surface to be created.
+   * @return the appropriate stride to use given the desired format and width, or
+   * -1 if either the format is invalid or the width too large.
+   *
+   * @since 1.6
+   **/
+  static int format_stride_for_width (Cairo::Format format, int width);
+
+  /**
+   * Creates an image surface of the specified format and dimensions. Initially
+   * the surface contents are all 0. (Specifically, within each pixel, each
+   * color or alpha channel belonging to format will be 0. The contents of bits
+   * within a pixel, but not belonging to the given format are undefined).
    *
    * @param format 	format of pixels in the surface to create
    * @param width 	width of the surface, in pixels
@@ -296,34 +412,34 @@ public:
    */
   static RefPtr<ImageSurface> create(Format format, int width, int height);
 
-  /** Creates an image surface for the provided pixel data. The output buffer
-   * must be kept around until the Surface is destroyed or finish() is called
-   * on the surface. The initial contents of buffer will be used as the inital
-   * image contents; you must explicitely clear the buffer, using, for example,
-   * Cairo::Context::rectangle() and Cairo::Context::fill() if you want it
-   * cleared.
+  /**
+   * Creates an image surface for the provided pixel data. The output buffer
+   * must be kept around until the surface is destroyed or finish() is called on
+   * the surface. The initial contents of buffer will be used as the initial
+   * image contents; you must explicitly clear the buffer, using, for example,
+   * Context::rectangle() and Context::fill() if you want it cleared.
    *
-   * If you want to be able to manually manipulate or extract the data after
-   * drawing to the surface with Cairo, you should use this function to create
-   * the Surface.  Since you own the internal data, you can do anything you
-   * want with it.
+   * Note that the stride may be larger than @a width * @a bytes_per_pixel to
+   * provide proper alignment for each pixel and row. This alignment is required
+   * to allow high-performance rendering within cairo. The correct way to obtain
+   * a legal stride value is to call format_stride_for_width() with the desired
+   * format and maximum image width value, and the use the resulting stride
+   * value to allocate the data and to create the image surface. See
+   * format_stride_for_width() for example code.
    *
-   * @param data	a pointer to a buffer supplied by the application in which
-   * to write contents.
-   * @param format	the format of pixels in the buffer
-   * @param width	the width of the image to be stored in the buffer
-   * @param height	the height of the image to be stored in the buffer
-   * @param stride	the number of bytes between the start of rows in the
-   * buffer. Having this be specified separate from width allows for padding at
-   * the end of rows, or for writing to a subportion of a larger image.
-   * @return	a RefPtr to the newly created surface.
+   * @param data a pointer to a buffer supplied by the application in which to write contents. This pointer must be suitably aligned for any kind of variable, (for example, a pointer returned by malloc).
+   * @param format the format of pixels in the buffer
+   * @param width the width of the image to be stored in the buffer
+   * @param height the height of the image to be stored in the buffer
+   * @param stride the number of bytes between the start of rows in the buffer as allocated. This value should always be computed by cairo_format_stride_for_width() before allocating the data buffer.
+   * @return a RefPtr to the newly created surface.
    */
   static RefPtr<ImageSurface> create(unsigned char* data, Format format, int width, int height, int stride);
 
 #ifdef CAIRO_HAS_PNG_FUNCTIONS
 
   /** Creates a new image surface and initializes the contents to the given PNG
-   * file.  
+   * file.
    *
    * @note For this function to be available, cairo must have been compiled
    * with PNG support.
@@ -335,16 +451,17 @@ public:
   static RefPtr<ImageSurface> create_from_png(std::string filename);
 
   /** Creates a new image surface from PNG data read incrementally via the
-   * read_func function.  
+   * read_func function.
    *
    * @note For this function to be available, cairo must have been compiled
    * with PNG support.
    *
-   * @param read_func	function called to read the data of the file
-   * @param closure	data to pass to read_func.
-   * @return	a RefPtr to the new cairo_surface_t initialized with the
+   * @param read_func function called to read the data of the file
+   * @return a RefPtr to the new cairo_surface_t initialized with the
    * contents of the PNG image file.
    */
+  static RefPtr<ImageSurface> create_from_png_stream(const SlotReadFunc& read_func);
+  /** @deprecated Use create_from_png_stream instead */
   static RefPtr<ImageSurface> create_from_png(cairo_read_func_t read_func, void *closure);
 
 #endif // CAIRO_HAS_PNG_FUNCTIONS
@@ -354,6 +471,10 @@ public:
 
 #ifdef CAIRO_HAS_PDF_SURFACE
 
+/** @example pdf-surface.cc
+ * An example of using Cairo::PdfSurface class to render to PDF
+ */
+
 /** A PdfSurface provides a way to render PDF documents from cairo.  This
  * surface is not rendered to the screen but instead renders the drawing to a
  * PDF file on disk.
@@ -389,10 +510,13 @@ public:
    *
    * @param write_func  The function to be called when the backend needs to
    * write data to an output stream
-   * @param closure     closure data for the write function
    * @param width_in_points   The width of the PDF document in points
    * @param height_in_points   The height of the PDF document in points
+   *
+   * @since 1.8
    */
+  static RefPtr<PdfSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
+  /** @deprecated use PdfSurface::create_for_stream() instead */
   static RefPtr<PdfSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
 
 /**
@@ -415,6 +539,19 @@ public:
 
 #ifdef CAIRO_HAS_PS_SURFACE
 
+/** @example ps-surface.cc
+ * An example of using Cairo::PsSurface class to render to PostScript
+ */
+
+/**
+ * describes the language level of the PostScript Language Reference that a
+ * generated PostScript file will conform to.
+ */
+typedef enum {
+    PS_LEVEL_2 = CAIRO_PS_LEVEL_2,
+    PS_LEVEL_3 = CAIRO_PS_LEVEL_3
+} PsLevel;
+
 /** A PsSurface provides a way to render PostScript documents from cairo.  This
  * surface is not rendered to the screen but instead renders the drawing to a
  * PostScript file on disk.
@@ -450,10 +587,13 @@ public:
    *
    * @param write_func  The function to be called when the backend needs to
    * write data to an output stream
-   * @param closure     closure data for the write function
    * @param width_in_points   The width of the PostScript document in points
    * @param height_in_points   The height of the PostScript document in points
+   *
+   * @since 1.8
    */
+  static RefPtr<PsSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
+  /** @deprecated use PsSurface::create_for_stream() instead */
   static RefPtr<PsSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
 
   /**
@@ -496,6 +636,60 @@ public:
    */
   void dsc_begin_page_setup();
 
+  /**
+   * If eps is true, the PostScript surface will output Encapsulated
+   * PostScript.
+   *
+   * This function should only be called before any drawing operations
+   * have been performed on the current page. The simplest way to do
+   * this is to call this function immediately after creating the
+   * surface. An Encapsulated Postscript file should never contain more
+   * than one page.
+   *
+   * @since 1.6
+   **/
+  void set_eps(bool eps);
+
+  /** Check whether the PostScript surface will output Encapsulated PostScript.
+   *
+   * @since 1.8
+   */
+  bool get_eps() const;
+
+  /**
+   * Restricts the generated PostSript file to @level. See get_levels() for a
+   * list of available level values that can be used here.
+   *
+   * This function should only be called before any drawing operations have been
+   * performed on the given surface. The simplest way to do this is to call this
+   * function immediately after creating the surface.
+   *
+   * @param level PostScript level
+   *
+   * @since 1.6
+   **/
+  void restrict_to_level(PsLevel level);
+
+  /**
+   * Used to retrieve the list of supported levels. See
+   * restrict_to_level().
+   *
+   * @since 1.6
+   **/
+  static const std::vector<PsLevel> get_levels();
+
+  /**
+   * Get the string representation of the given level id. This function will
+   * return an empty string if level id isn't valid. See get_levels() for a way
+   * to get the list of valid level ids.
+   *
+   * @return the string associated to given level.
+   *
+   * @param level a level id
+   *
+   * @since 1.6
+   **/
+  static std::string level_to_string(PsLevel level);
 };
 
 #endif // CAIRO_HAS_PS_SURFACE
@@ -503,6 +697,10 @@ public:
 
 #ifdef CAIRO_HAS_SVG_SURFACE
 
+/** @example svg-surface.cc
+ * An example of using Cairo::SvgSurface class to render to SVG
+ */
+
 typedef enum
 {
   SVG_VERSION_1_1 = CAIRO_SVG_VERSION_1_1,
@@ -545,10 +743,14 @@ public:
    *
    * @param write_func  The function to be called when the backend needs to
    * write data to an output stream
-   * @param closure     closure data for the write function
    * @param width_in_points   The width of the SVG document in points
    * @param height_in_points   The height of the SVG document in points
+   *
+   * @since 1.8
    */
+  static RefPtr<SvgSurface> create_for_stream(const SlotWriteFunc& write_func, double width_in_points, double height_in_points);
+
+  /** @deprecated Use SvgSurface::create_for_stream() instead */
   static RefPtr<SvgSurface> create(cairo_write_func_t write_func, void *closure, double width_in_points, double height_in_points);
 
   /** 
@@ -574,7 +776,7 @@ public:
    * will be empty if version isn't valid. See get_versions() for a way to get
    * the list of valid version ids.
    *
-   * Since: 1.2
+   * since: 1.2
    */
   static std::string version_to_string(SvgVersion version);
 };
